Package: statsapps
Title: Interactive 'Shiny' Apps for Building Statistical Intuition
Version: 0.1.0
Authors@R: 
    person(given = "Vikram B.",
                 family = "Baliga", 
                 role = c("aut", "cre", "cph"),
                 email = "vikram.baliga@ubc.ca",
                 comment = c(ORCID = "0000-0002-9367-8974")
                 )
Description: 
    Provides interactive 'Shiny' applications for building intuition in 
    statistics and data science. The apps run locally and use
    visualization, simulation, and repeated sampling to illustrate topics
    such as probability distributions, permutation tests, analysis of
    variance, linear regression, and sums of squares.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 8.0.0
Imports: dplyr (>= 1.1.0), ggplot2 (>= 3.4.0), purrr (>= 1.0.0), shiny
        (>= 1.7.0), tibble (>= 3.0.0)
Depends: R (>= 4.1.0)
URL: https://github.com/vbaliga/statsapps,
        https://vbaliga.github.io/statsapps/
BugReports: https://github.com/vbaliga/statsapps/issues
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-16 02:11:38 UTC; vikram
Author: Vikram B. Baliga [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-9367-8974>)
Maintainer: Vikram B. Baliga <vikram.baliga@ubc.ca>
Repository: CRAN
Date/Publication: 2026-07-23 13:30:02 UTC
